/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLDocument.java,v 1.1.1.1 2006/01/31 09:13:25 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser;

import java.io.InputStream;

import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;

/**
 * OWLDocument - OWL otology document.
 * 
 * OWL document (local file or web resource with linked uri) describes an OWL
 * ontology in which all elements and statements of the ontology are defined.
 *  
 */
public interface OWLDocument {
    /**
     * Check if OWL document is a local file or a web resource with linked uri.
     * 
     * @return true if the OWL document is a local file
     */
    boolean isLocalFile();

    /**
     * Get the document local uri. <br>
     * If OWL document is a local file, the current file path is used as the
     * document's local uri.
     * 
     * @return the document local uri
     */
    String getLocalURI();

    /**
     * Get the document public uri. <br>
     * If OWL document is a web resource, the web uri is used as the document's
     * public uri.
     * 
     * @return the document public uri
     */
    String getPublicURI();

    /**
     * Get OWL Document charset(encoding).
     * 
     * @return the charset
     */
    String getCharset();

    /**
     * Set OWL Document charset(encoding).
     * 
     * @param charset
     *            the charset
     */
    void setCharset(String charset);

    /**
     * Get default base uri.
     * 
     * @return default base uri
     */
    String getDefaultBaseURI();

    /**
     * Set default base uri.
     * 
     * @param uri
     *            default base uri
     */
    void setDefaultBaseURI(String uri);

    /**
     * Read the OWL document from the input.
     * 
     * @return the input stream
     * @exception OWLParserException
     *                if io error
     */
    InputStream readInput() throws OWLParserException;

    /**
     * Set public URI of the document
     * 
     * @param URI
     *            the URI of the document
     */
    void setPublicURI(String URI);
}