/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLModelDocMappingImpl.java,v 1.2 2006/07/19 10:09:19 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.eodm.owl.resource.parser.OWLDocument;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.OWLOntoModel;
import org.eclipse.eodm.rdf.resource.parser.element.RDFTriple;
import org.eclipse.eodm.rdfs.RDFList;

/**
 * OWLModelDocMappingImpl
 */
public class OWLModelDocMappingImpl implements OWLModelDocMapping {
    /**
     * Constructs an OWLModelDocMappingImpl instance.
     */
    public OWLModelDocMappingImpl(OWLOntoModel model) {
        super();
        this.model = model;
    }

    /** The ontology model */
    private OWLOntoModel model = null;

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getModel()
     */
    public OWLOntoModel getModel() {
        return model;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#setModel(org.eclipse.owl.parser.mem.OWLOntoModel)
     */
    public void setModel(OWLOntoModel model) {
        this.model = model;
    }

    /** documents related to the ontology model */
    private List docs = new ArrayList();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#addDoc(org.eclipse.owl.parser.mem.OWLDocument)
     */
    public void addDoc(OWLDocument doc) {
        docs.add(doc);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getDocByLURI(java.lang.String)
     */
    public OWLDocument getDocByLURI(String lURI) {
        if (lURI == null)
            return null;
        for (int i = 0; i < docs.size(); i++) {
            OWLDocument document = (OWLDocument) docs.get(i);
            if (lURI.equals(document.getLocalURI()))
                return document;
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getDocByPURI(java.lang.String)
     */
    public OWLDocument getDocByPURI(String pURI) {
        if (pURI == null)
            return null;
        for (int i = 0; i < docs.size(); i++) {
            OWLDocument document = (OWLDocument) docs.get(i);
            if (pURI.equals(document.getPublicURI()))
                return document;
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getDocs()
     */
    public List getDocs() {
        return docs;
    }

    /** All the restrictions with ambiguous type */
    Map restrictions = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#addRestriction(java.lang.String)
     */
    public void addRestriction(String id) {
        if (id == null)
            return;
        if (!restrictions.containsKey(id))
            restrictions.put(id, id);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#containsRestriction(java.lang.String)
     */
    public boolean containsRestriction(String id) {
        return restrictions.containsKey(id);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getRestrictions()
     */
    public Map getRestrictions() {
        return restrictions;
    }

    /** All the lists */
    Map lists = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#addList(org.eclipse.rdfs.RDFList)
     */
    public void addList(RDFList list) {
        if (list == null || list.getLocalName() == null)
            return;
        if (!lists.containsKey(list.getLocalName()))
            lists.put(list.getLocalName(), list);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getList(java.lang.String)
     */
    public RDFList getList(String id) {
        return (RDFList) lists.get(id);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getLists()
     */
    public Map getLists() {
        return lists;
    }

    /** All imported model uris */
    Map impURIs = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#addImpURI(java.lang.String)
     */
    public void addImpURI(String impURI) {
        if (impURI == null)
            return;
        if (!impURIs.containsKey(impURI))
            impURIs.put(impURI, impURI);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getImpURIs()
     */
    public Map getImpURIs() {
        return impURIs;
    }

    /** All unparsed triples */
    List unparsedTriples = new Vector();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#addUnparsedTriple(org.eclipse.rdfs.resource.parser.element.RDFTriple)
     */
    public void addUnparsedTriple(RDFTriple triple) {
        unparsedTriples.add(triple);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getUnparsedTriples()
     */
    public List getUnparsedTriples() {
        return unparsedTriples;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#clearResources()
     */
    public void clearResources() {
        setModel(null);
        docs.clear();
        restrictions.clear();
        lists.clear();
        impURIs.clear();
        references.clear();
        unparsedTriples.clear();
    }

    List references = new Vector();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#addAxiom(org.eclipse.owl.resource.parser.impl.OWLAxiom)
     */
    public void addReference(ReferenceRecorder reference) {
        references.add(reference);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocMapping#getAxioms()
     */
    public List getReferences() {
        return references;
    }
}