/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLModelDocRepositoryImpl.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.eodm.owl.resource.parser.OWLDocument;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocRepository;
import org.eclipse.eodm.owl.resource.parser.OWLOntoModel;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;

/**
 * OWLModelDocRepositoryImpl
 *  
 */
public class OWLModelDocRepositoryImpl implements OWLModelDocRepository {
    /**
     * Constructs an OWL Ontology Repository instance.
     */
    public OWLModelDocRepositoryImpl() {
        super();
    }

    /** all the ontology models */
    private Map ontoModels = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#addOntoModel(org.eclipse.owl.parser.mem.OWLOntoModel)
     */
    public void addOntoModel(OWLOntoModel ontoModel) {
        if (ontoModel == null || ontoModel.getModelURI() == null)
            return;
        if (!ontoModels.containsKey(ontoModel.getModelURI()))
            ontoModels.put(ontoModel.getModelURI(), ontoModel);
        else
            throw new OWLParserException("The ontology model "
                                         + ontoModel.getModelURI()
                                         + " already exists.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getOntoModel(java.lang.String)
     */
    public OWLOntoModel getOntoModel(String modelURI) {
        return (OWLOntoModel) ontoModels.get(modelURI);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getAllOntoModels()
     */
    public Map getAllOntoModels() {
        return ontoModels;
    }

    /** all the ontology model-documents mappings */
    private Map modelDocMappings = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#addModelDocMapping(org.eclipse.owl.parser.mem.OWLModelDocMapping)
     */
    public void addModelDocMapping(OWLModelDocMapping modelDocMapping) {
        if (!modelDocMappings.containsKey(modelDocMapping.getModel()
                .getModelURI()))
            modelDocMappings.put(modelDocMapping.getModel().getModelURI(),
                    modelDocMapping);
        else
            throw new OWLParserException("The model - document mapping "
                                         + modelDocMapping.getModel()
                                                 .getModelURI()
                                         + " already exists.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getMappingByModelURI(java.lang.String)
     */
    public OWLModelDocMapping getMappingByModelURI(String modelURI) {
        return (OWLModelDocMapping) modelDocMappings.get(modelURI);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getMappingByDocument(org.eclipse.owl.parser.mem.OWLDocument)
     */
    public OWLModelDocMapping getMappingByDocument(OWLDocument document) {
        Iterator keys = modelDocMappings.keySet().iterator();
        while (keys.hasNext()) {
            OWLModelDocMapping mapping = (OWLModelDocMapping) modelDocMappings
                    .get(keys.next());
            if (mapping.getDocByLURI(document.getLocalURI()) != null)
                return mapping;
            if (mapping.getDocByPURI(document.getPublicURI()) != null)
                return mapping;
        }
        return null;
    }

    /** all the documents */
    private List documents = new ArrayList();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#addDocument(org.eclipse.owl.parser.mem.OWLDocument)
     */
    public void addDocument(OWLDocument document) {
        if (getDocumentByLocalURI(document.getLocalURI()) != null
            || getDocumentByLocalURI(document.getPublicURI()) != null
            || getDocumentByPublicURI(document.getLocalURI()) != null
            || getDocumentByPublicURI(document.getPublicURI()) != null)
            throw new OWLParserException(
                    "Document "
                                                                                + (document
                                                                                        .getLocalURI() == null ? document
                                                                                        .getPublicURI()
                                                                                        : document
                                                                                                .getLocalURI())
                                                                                + " already exists.");
        documents.add(document);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getDocumentByLocalURI(java.lang.String)
     */
    public OWLDocument getDocumentByLocalURI(String localURI) {
        if (localURI == null)
            return null;
        for (int i = 0; i < documents.size(); i++) {
            OWLDocument document = (OWLDocument) documents.get(i);
            if (localURI.equals(document.getLocalURI()))
                return document;
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getDocumentByPublicURI(java.lang.String)
     */
    public OWLDocument getDocumentByPublicURI(String publicURI) {
        if (publicURI == null)
            return null;
        for (int i = 0; i < documents.size(); i++) {
            OWLDocument document = (OWLDocument) documents.get(i);
            if (publicURI.equals(document.getPublicURI()))
                return document;
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getAllDocuments()
     */
    public List getAllDocuments() {
        return documents;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#clearResources()
     */
    public void clearResources() {
        documents.clear();
        Iterator modelKeys = ontoModels.keySet().iterator();
        while (modelKeys.hasNext())
            ((OWLOntoModel) ontoModels.get(modelKeys.next())).clearResources();
        ontoModels.clear();
        Iterator modelDocMappingKeys = modelDocMappings.keySet().iterator();
        while (modelDocMappingKeys.hasNext())
            ((OWLModelDocMapping) modelDocMappings.get(modelDocMappingKeys
                    .next())).clearResources();
        modelDocMappings.clear();
        references.clear();
    }

    List references = new Vector();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#addReference(org.eclipse.owl.resource.parser.impl.ReferenceRecorder)
     */
    public void addReference(ReferenceRecorder reference) {
        references.add(reference);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLModelDocRepository#getReferences()
     */
    public List getReferences() {
        return references;
    }
}