/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLOntoModelImpl.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.eodm.owl.Individual;
import org.eclipse.eodm.owl.OWLAllDifferent;
import org.eclipse.eodm.owl.OWLAnnotationProperty;
import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLDataRange;
import org.eclipse.eodm.owl.OWLDatatypeProperty;
import org.eclipse.eodm.owl.OWLObjectProperty;
import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.OWLOntologyProperty;
import org.eclipse.eodm.owl.OWLRestriction;
import org.eclipse.eodm.owl.resource.parser.OWLOntoModel;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.RDFSContainer;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFStatement;

/**
 * OWLOntoModelImpl
 *  
 */
public class OWLOntoModelImpl implements OWLOntoModel {
    /**
     * Constructs an OWLOntoModelImpl instance.
     */
    public OWLOntoModelImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getModelURI()
     */
    public String getModelURI() {
        return getOntology() == null ? null : getOntology().getURI();
    }

    /** Indicating whether the model has been parsed completely */
    private boolean parsed = false;

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#isParsed()
     */
    public boolean isParsed() {
        return parsed;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#setParsed(boolean)
     */
    public void setParsed(boolean parsed) {
        this.parsed = parsed;
    }

    /** The ontology of the model */
    OWLOntology ontology = null;

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#setOntology(org.eclipse.owl.OWLOntology)
     */
    public void setOntology(OWLOntology ontology) {
        this.ontology = ontology;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getOntology()
     */
    public OWLOntology getOntology() {
        return ontology;
    }

    /** All the namespaces defined in the documents */
    private Map namespaces = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addNamespace(org.eclipse.rdfs.Namespace)
     */
    public void addNamespace(Namespace namespace) {

        if (namespace == null || namespace.getURI() == null)
            return;

        if (!namespaces.containsKey(namespace.getURI())) {
            namespaces.put(namespace.getURI(), namespace);
            ontology.getOwnedNamespace().add(namespace);
        }else
        {
            Namespace ns=(Namespace) namespaces.get( namespace.getURI());
            if (ns.getName().startsWith("ns_") && !namespace.getName().startsWith("ns_"))
                ns.setName(namespace.getName());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getNamespace(java.lang.String)
     */
    public Namespace getNamespace(String uri) {
        Namespace ns = (Namespace) namespaces.get(uri);
        if (ns == null) {
            ns = RDFTypedElementGenerator._INSTANCE.createNamespace(null, uri);
            addNamespace(ns);
        }
        return ns;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getNamespaces()
     */
    public Map getNamespaces() {
        return namespaces;
    }

    /** All the imported models defined in the documents */
    private Map impModels = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addImpModel(org.eclipse.owl.parser.mem.OWLOntoModel)
     */
    public void addImpModel(OWLOntoModel model) {
        if (model == null || model.getModelURI() == null)
            return;
        if (!impModels.containsKey(model.getModelURI())) {
            impModels.put(model.getModelURI(), model);
            ontology.getOWLImports().add(model.getOntology());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getImpModel(java.lang.String)
     */
    public OWLOntoModel getImpModel(String uri) {
        return (OWLOntoModel) impModels.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getImpModels()
     */
    public Map getImpModels() {
        return impModels;
    }

    /** All the classes */
    private Map classes = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addClass(org.eclipse.owl.OWLClass)
     */
    public void addClass(OWLClass cls) {
        if (cls == null || cls.getURI() == null)
            return;
        if (!classes.containsKey(cls.getURI())) {
            classes.put(cls.getURI(), cls);
            ontology.getContains().add(cls);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getClass(java.lang.String)
     */
    public OWLClass getClass(String uri) {
        return (OWLClass) classes.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getClasses()
     */
    public Map getClasses() {
        return classes;
    }

    /** All the restrictions */
    private Map restrictions = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addRestriction(org.eclipse.owl.OWLRestriction)
     */
    public void addRestriction(OWLRestriction restriction) {
        if (restriction == null || restriction.getURI() == null)
            return;
        if (!restrictions.containsKey(restriction.getURI())) {
            restrictions.put(restriction.getURI(), restriction);
            ontology.getContains().add(restriction);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getRestriction(java.lang.String)
     */
    public OWLRestriction getRestriction(String uri) {
        return (OWLRestriction) restrictions.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getRestrictions()
     */
    public Map getRestrictions() {
        return restrictions;
    }

    /** All the object properties */
    private Map objectProperties = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addObjectProperty(org.eclipse.owl.OWLObjectProperty)
     */
    public void addObjectProperty(OWLObjectProperty objectProperty) {
        if (objectProperty == null || objectProperty.getURI() == null)
            return;
        if (!objectProperties.containsKey(objectProperty.getURI())) {
            objectProperties.put(objectProperty.getURI(), objectProperty);
            ontology.getContains().add(objectProperty);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getObjectProperty(java.lang.String)
     */
    public OWLObjectProperty getObjectProperty(String uri) {
        return (OWLObjectProperty) objectProperties.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getObjectProperties()
     */
    public Map getObjectProperties() {
        return objectProperties;
    }

    /** All the datatype properties */
    private Map datatypeProperties = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addDatatypeProperty(org.eclipse.owl.OWLDatatypeProperty)
     */
    public void addDatatypeProperty(OWLDatatypeProperty datatypeProperty) {
        if (datatypeProperty == null || datatypeProperty.getURI() == null)
            return;
        if (!datatypeProperties.containsKey(datatypeProperty.getURI())) {
            datatypeProperties.put(datatypeProperty.getURI(), datatypeProperty);
            ontology.getContains().add(datatypeProperty);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getDatatypeProperty(java.lang.String)
     */
    public OWLDatatypeProperty getDatatypeProperty(String uri) {
        return (OWLDatatypeProperty) datatypeProperties.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getDatatypeProperties()
     */
    public Map getDatatypeProperties() {
        return datatypeProperties;
    }

    /** All the individuals */
    private Map individuals = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addIndividual(org.eclipse.owl.Individual)
     */
    public void addIndividual(Individual individual) {
        if (individual == null || individual.getURI() == null)
            return;
        if (!individuals.containsKey(individual.getURI())) {
            individuals.put(individual.getURI(), individual);
            ontology.getContains().add(individual);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getIndividual(java.lang.String)
     */
    public Individual getIndividual(String uri) {
        return (Individual) individuals.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getIndividuals()
     */
    public Map getIndividuals() {
        return individuals;
    }

    /** All the dataRanges */
    private Map dataRanges = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addDataRange(org.eclipse.owl.OWLDataRange)
     */
    public void addDataRange(OWLDataRange dataRange) {
        if (dataRange == null || dataRange.getURI() == null)
            return;
        if (!dataRanges.containsKey(dataRange.getURI())) {
            dataRanges.put(dataRange.getURI(), dataRange);
            ontology.getContains().add(dataRange);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getDataRange(java.lang.String)
     */
    public OWLDataRange getDataRange(String uri) {
        return (OWLDataRange) dataRanges.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getDataRanges()
     */
    public Map getDataRanges() {
        return dataRanges;
    }

    /** All the allDifferents */
    private Map allDifferents = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addAllDifferent(java.lang.String,
     *      org.eclipse.owl.OWLAllDifferent)
     */
    public void addAllDifferent(String id, OWLAllDifferent allDifferent) {
        if (id == null || allDifferent == null)
            return;
        if (!allDifferents.containsKey(id))
            allDifferents.put(id, allDifferent);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getAllDifferent(java.lang.String)
     */
    public OWLAllDifferent getAllDifferent(String id) {
        return (OWLAllDifferent) allDifferents.get(id);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getAllDifferents()
     */
    public Map getAllDifferents() {
        return allDifferents;
    }

    /** All the Statements */
    Map statements = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addStatement(org.eclipse.rdfs.RDFStatement)
     */
    public void addStatement(RDFStatement statement) {
        if (statement == null || statement.getURI() == null)
            return;
        if (!statements.containsKey(statement.getURI())) {
            statements.put(statement.getURI(), statement);
            ontology.getContains().add(statement);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getStatement(java.lang.String)
     */
    public RDFStatement getStatement(String uri) {
        return (RDFStatement) statements.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getStatements()
     */
    public Map getStatements() {
        return statements;
    }

    /** All the datatypes */
    Map datatypes = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addDatatype(org.eclipse.rdfs.RDFSDatatype)
     */
    public void addDatatype(RDFSDatatype datatype) {
        if (datatype == null || datatype.getURI() == null)
            return;
        if (!datatypes.containsKey(datatype.getURI())) {
            datatypes.put(datatype.getURI(), datatype);
            ontology.getContains().add(datatype);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getDatatype(java.lang.String)
     */
    public RDFSDatatype getDatatype(String uri) {
        return (RDFSDatatype) datatypes.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getDatatypes()
     */
    public Map getDatatypes() {
        return datatypes;
    }

    /** All the containers */
    Map containers = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addContainer(org.eclipse.rdfs.RDFSContainer)
     */
    public void addContainer(RDFSContainer container) {
        if (container == null || container.getURI() == null)
            return;
        if (!containers.containsKey(container.getURI())) {
            containers.put(container.getURI(), container);
            ontology.getContains().add(container);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getContainer(java.lang.String)
     */
    public RDFSContainer getContainer(String uri) {
        return (RDFSContainer) containers.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getContainers()
     */
    public Map getContainers() {
        return containers;
    }

    /** Some other ontologies */
    private Map otherOntos = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addOtherOnto(org.eclipse.owl.OWLOntology)
     */
    public void addOtherOnto(OWLOntology otherOnto) {
        if (otherOnto == null || otherOnto.getURI() == null)
            return;
        if (!otherOntos.containsKey(otherOnto.getURI())) {
            otherOntos.put(otherOnto.getURI(), otherOnto);
            ontology.getContains().add(otherOnto);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getOtherOnto(java.lang.String)
     */
    public OWLOntology getOtherOnto(String uri) {
        return (OWLOntology) otherOntos.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getResource(java.lang.String)
     */
    public RDFSResource getResource(String uri) {
        RDFSResource result = null;
        if (ontology.getURI().equals(uri))
            return ontology;
        result = getClass(uri);
        if (result != null)
            return result;
        result = getRestriction(uri);
        if (result != null)
            return result;
        result = getObjectProperty(uri);
        if (result != null)
            return result;
        result = getDatatypeProperty(uri);
        if (result != null)
            return result;
        result = getIndividual(uri);
        if (result != null)
            return result;
        result = getDataRange(uri);
        if (result != null)
            return result;
        result = getStatement(uri);
        if (result != null)
            return result;
        result = getDatatype(uri);
        if (result != null)
            return result;
        result = getContainer(uri);
        if (result != null)
            return result;
        result = getRDFSResource(uri);
        if (result != null)
            return result;
        result = this.getAnnoProperty(uri);
        if (result != null)
            return result;
        result = this.getOntoProperty(uri);
        if (result != null)
            return result;
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#clearResources()
     */
    public void clearResources() {
        namespaces.clear();
        impModels.clear();
        setOntology(null);
        classes.clear();
        restrictions.clear();
        objectProperties.clear();
        datatypeProperties.clear();
        individuals.clear();
        dataRanges.clear();
        allDifferents.clear();
        statements.clear();
        datatypes.clear();
        containers.clear();
        otherOntos.clear();
        annoProperties.clear();
        ontoProperties.clear();
        RDFSResources.clear();
    }

    Map annoProperties = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addAnnoProperty(org.eclipse.owl.OWLAnnotationProperty)
     */
    public void addAnnoProperty(OWLAnnotationProperty annoProperty) {
        if (annoProperty == null || annoProperty.getURI() == null)
            return;
        if (!annoProperties.containsKey(annoProperty.getURI())) {
            annoProperties.put(annoProperty.getURI(), annoProperty);
            ontology.getContains().add(annoProperty);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getAnnoProperty(java.lang.String)
     */
    public OWLAnnotationProperty getAnnoProperty(String uri) {
        return (OWLAnnotationProperty) annoProperties.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getAnnoProperties()
     */
    public Map getAnnoProperties() {
        return annoProperties;
    }

    Map ontoProperties = new HashMap();

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addOntoProperty(org.eclipse.owl.OWLOntologyProperty)
     */
    public void addOntoProperty(OWLOntologyProperty ontoProperty) {
        if (ontoProperty == null || ontoProperty.getURI() == null)
            return;
        if (!ontoProperties.containsKey(ontoProperty.getURI())) {
            ontoProperties.put(ontoProperty.getURI(), ontoProperty);
            ontology.getContains().add(ontoProperty);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getOntoProperty(java.lang.String)
     */
    public OWLOntologyProperty getOntoProperty(String uri) {
        return (OWLOntologyProperty) ontoProperties.get(uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getOntoProperties()
     */
    public Map getOntoProperties() {
        return ontoProperties;
    }

    Map RDFSResources = new HashMap();
    /* (non-Javadoc)
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#addRDFSResource(org.eclipse.rdfs.RDFSResource)
     */
    public void addRDFSResource(RDFSResource resource) {
        if (resource == null || resource == null)
            return;
        if (!RDFSResources.containsKey(resource.getURI())) {
            RDFSResources.put(resource.getURI(), resource);
            ontology.getContains().add(resource);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getRDFSResource(java.lang.String)
     */
    public RDFSResource getRDFSResource(String uri) {
        return (RDFSResource) RDFSResources.get(uri);
    }

    /* (non-Javadoc)
     * @see org.eclipse.owl.resource.parser.wrapper.OWLOntoModel#getRDFSResource()
     */
    public Map getRDFSResource() {        
        return RDFSResources;
    }
}